from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User


class RegistrationForm(UserCreationForm):

    first_name = forms.CharField(max_length=30, help_text="Required")
    last_name = forms.CharField(max_length=30, help_text="Required")
    email = forms.EmailField(max_length=254, help_text="Required")

    class Meta:
        model = User
        fields = (
            "username",
            "first_name",
            "last_name",
            "email",
            "password1",
            "password2",
        )



# Generated by Django 2.2.20 on 2021-07-21 17:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0061_outcomehorizontallink_degree"),
    ]

    operations = [
        migrations.AlterField(
            model_name="favourite",
            name="content_type",
            field=models.ForeignKey(
                limit_choices_to={
                    "model__in": ["project", "activity", "course", "program"]
                },
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AlterField(
            model_name="objectpermission",
            name="content_type",
            field=models.ForeignKey(
                limit_choices_to={
                    "model__in": ["project", "activity", "course", "program"]
                },
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.ContentType",
            ),
        ),
    ]



from smtplib import SMTPException

import pandas as pd
from celery import shared_task
from django.conf import settings
from django.core.cache import cache
from django.core.files.storage import default_storage
from django.core.mail import EmailMessage
from django.utils import timezone

from course_flow import export_functions, import_functions
from course_flow import redux_actions as actions

from .celery import logger, try_async
from .models import ObjectSet, User
from .utils import dateTimeFormatNoSpace, get_model_from_str


@try_async
@shared_task
def async_send_export_email(
    user_email,
    pk,
    object_type,
    export_type,
    export_format,
    allowed_sets,
    email_subject,
    email_text,
):
    model_object = get_model_from_str(object_type).objects.get(pk=pk)
    if object_type == "project":
        project_sets = ObjectSet.objects.filter(project=model_object)
    else:
        project_sets = ObjectSet.objects.filter(
            project=model_object.get_project()
        )
    allowed_sets = project_sets.filter(id__in=allowed_sets)
    if export_type == "outcome":
        file = export_functions.get_outcomes_export(
            model_object, object_type, export_format, allowed_sets
        )
    elif export_type == "framework":
        file = export_functions.get_course_frameworks_export(
            model_object, object_type, export_format, allowed_sets
        )
    elif export_type == "matrix":
        file = export_functions.get_program_matrix_export(
            model_object, object_type, export_format, allowed_sets
        )
    elif export_type == "node":
        file = export_functions.get_nodes_export(
            model_object, object_type, export_format, allowed_sets
        )
    if export_format == "excel":
        file_ext = "xlsx"
    elif export_format == "csv":
        file_ext = "csv"

    filename = (
        object_type
        + "_"
        + str(pk)
        + "_"
        + export_type
        + "_"
        + timezone.now().strftime(dateTimeFormatNoSpace())
        + "."
        + file_ext
    )
    email = EmailMessage(
        email_subject,
        email_text,
        settings.DEFAULT_FROM_EMAIL,
        [user_email],
    )
    if export_format == "csv":
        file_data = "text/csv"
    elif export_format == "excel":
        file_data = (
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        )
    if settings.DEBUG:
        with open("last_export." + file_ext, "wb") as out_file:
            out_file.write(file)

    email.attach(
        filename,
        file,
        file_data,
    )
    try:
        email.send()
        logger.info(
            f"Email - {email_subject} - {filename} - sent to {user_email}"
        )
    except SMTPException:
        logger.info(
            f"Email - {email_subject} - {filename} - could NOT be sent to {user_email}"
        )


@try_async
@shared_task
def async_import_file_data(pk, object_type, task_type, file_json, user_id):
    model_object = get_model_from_str(object_type).objects.get(pk=pk)
    user = User.objects.get(pk=user_id)
    if object_type == "workflow":
        actions.dispatch_wf(
            model_object,
            actions.changeField(pk, "workflow", {"importing": True}, False),
        )
    cache.set(object_type + str(pk) + "importing", True, 300)
    df = pd.read_json(file_json)
    try:
        if task_type == "outcomes":
            import_functions.import_outcomes(df, model_object, user)
        if task_type == "nodes":
            import_functions.import_nodes(df, model_object, user)
    except Exception:
        pass
    cache.delete(object_type + str(pk) + "importing")
    if object_type == "workflow":
        actions.dispatch_wf(
            model_object,
            actions.changeField(pk, "workflow", {"importing": False}, False),
        )



# Generated by Django 3.2.15 on 2023-01-18 22:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0100_courseflowuser_notifications"),
    ]

    operations = [
        migrations.AddField(
            model_name="courseflowuser",
            name="notifications_active",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="courseflowuser",
            name="notifications",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.2.25 on 2022-03-22 21:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0082_auto_20220321_1804"),
    ]

    operations = [
        migrations.RenameField(
            model_name="objectset",
            old_name="translation",
            new_name="title",
        ),
    ]



import re
import time

from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.http import HttpResponse, JsonResponse
from django.urls import reverse

from course_flow import models

owned_throughmodels = [
    "node",
    "nodeweek",
    "week",
    "weekworkflow",
    "workflow",
    "workflowproject",
    "project",
    "columnworkflow",
    "workflow",
    "workflowproject",
    "project",
    "outcome",
    "outcomeoutcome",
    "outcome",
]


def get_alphanum(string):
    return re.sub(r"\W+", "", string)


# Create a regex from dict keys
def multiple_replace(dict, text):
    regex = re.compile("(%s)" % "|".join(map(re.escape, dict.keys())))
    return regex.sub(lambda mo: dict[mo.string[mo.start() : mo.end()]], text)


def dateTimeFormat():
    return "%Y/%m/%d"


def dateTimeFormatNoSpace():
    return "%Y_%m_%d_%H_%m_%s"


def get_model_from_str(model_str: str):
    return ContentType.objects.get(model=model_str).model_class()


def get_parent_model_str(model_str: str) -> str:

    return owned_throughmodels[owned_throughmodels.index(model_str) + 1]


def get_parent_model(model_str: str):

    return ContentType.objects.get(
        model=get_parent_model_str(model_str)
    ).model_class()


def linkIDMap(link):
    return link.id


def get_descendant_outcomes(outcome):
    return models.Outcome.objects.filter(
        Q(parent_outcomes=outcome)
        | Q(parent_outcomes__parent_outcomes=outcome)
    )


def get_all_outcomes_for_outcome(outcome):
    outcomes = models.Outcome.objects.filter(
        Q(parent_outcomes=outcome)
        | Q(parent_outcomes__parent_outcomes=outcome)
    ).prefetch_related(
        "outcome_horizontal_links", "child_outcome_links", "sets"
    )
    outcomeoutcomes = models.OutcomeOutcome.objects.filter(
        Q(parent=outcome) | Q(parent__parent_outcomes=outcome)
    )
    return outcomes, outcomeoutcomes


def get_all_outcomes_for_workflow(workflow):
    outcomes = models.Outcome.objects.filter(
        Q(workflow=workflow)
        | Q(parent_outcomes__workflow=workflow)
        | Q(parent_outcomes__parent_outcomes__workflow=workflow)
    ).prefetch_related(
        "outcome_horizontal_links", "child_outcome_links", "sets"
    )
    outcomeoutcomes = models.OutcomeOutcome.objects.filter(
        Q(parent__workflow=workflow)
        | Q(parent__parent_outcomes__workflow=workflow)
    )
    return outcomes, outcomeoutcomes


def get_all_outcomes_ordered_for_outcome(outcome):
    outcomes = [outcome]
    for outcomeoutcome in outcome.child_outcome_links.filter(
        child__deleted=False
    ).order_by("rank"):
        outcomes += get_all_outcomes_ordered_for_outcome(outcomeoutcome.child)
    return outcomes


def get_all_outcomes_ordered(workflow):
    outcomes = []
    for outcomeworkflow in workflow.outcomeworkflow_set.filter(
        outcome__deleted=False
    ).order_by("rank"):
        outcomes += get_all_outcomes_ordered_for_outcome(
            outcomeworkflow.outcome
        )
    return outcomes


def get_all_outcomes_ordered_filtered(workflow, extra_filter):
    outcomes = []
    for outcome in (
        models.Outcome.objects.filter(workflow=workflow, deleted=False)
        .filter(extra_filter)
        .order_by("outcomeworkflow__rank")
    ):
        outcomes += get_all_outcomes_ordered_for_outcome(outcome)
    return outcomes


def get_unique_outcomenodes(node):
    return (
        node.outcomenode_set.exclude(
            Q(outcome__deleted=True)
            | Q(outcome__parent_outcomes__deleted=True)
            | Q(outcome__parent_outcomes__parent_outcomes__deleted=True)
        )
        .exclude(outcome__parent_outcomes__node=node)
        .exclude(outcome__parent_outcomes__parent_outcomes__node=node)
        .order_by(
            "outcome__parent_outcome_links__parent__parent_outcome_links__parent__outcomeworkflow__rank",
            "outcome__parent_outcome_links__parent__outcomeworkflow__rank",
            "outcome__outcomeworkflow__rank",
            "outcome__parent_outcome_links__parent__parent_outcome_links__rank",
            "outcome__parent_outcome_links__rank",
        )
    )


def get_outcomenodes(node):
    return node.outcomenode_set.exclude(
        Q(outcome__deleted=True)
        | Q(outcome__parent_outcomes__deleted=True)
        | Q(outcome__parent_outcomes__parent_outcomes__deleted=True)
    ).order_by(
        "outcome__parent_outcome_links__parent__parent_outcome_links__parent__outcomeworkflow__rank",
        "outcome__parent_outcome_links__parent__outcomeworkflow__rank",
        "outcome__outcomeworkflow__rank",
        "outcome__parent_outcome_links__parent__parent_outcome_links__rank",
        "outcome__parent_outcome_links__rank",
    )


def get_unique_outcomehorizontallinks(outcome):
    return (
        outcome.outcome_horizontal_links.exclude(
            Q(parent_outcome__deleted=True)
            | Q(parent_outcome__parent_outcomes__deleted=True)
            | Q(parent_outcome__parent_outcomes__parent_outcomes__deleted=True)
        )
        .exclude(
            parent_outcome__parent_outcomes__reverse_horizontal_outcomes=outcome
        )
        .exclude(
            parent_outcome__parent_outcomes__parent_outcomes__reverse_horizontal_outcomes=outcome
        )
        .order_by(
            "parent_outcome__parent_outcome_links__parent__parent_outcome_links__parent__outcomeworkflow__rank",
            "parent_outcome__parent_outcome_links__parent__outcomeworkflow__rank",
            "parent_outcome__outcomeworkflow__rank",
            "parent_outcome__parent_outcome_links__parent__parent_outcome_links__rank",
            "parent_outcome__parent_outcome_links__rank",
        )
    )


def get_parent_nodes_for_workflow(workflow):
    nodes = (
        models.Node.objects.filter(linked_workflow=workflow)
        .exclude(
            Q(deleted=True)
            | Q(week__deleted=True)
            | Q(week__workflow__deleted=True)
        )
        .prefetch_related("outcomenode_set")
    )
    return nodes


def get_nondeleted_favourites(user):
    return list(
        models.Project.objects.filter(favourited_by__user=user)
    ) + list(models.Workflow.objects.filter(favourited_by__user=user))

    # return models.Favourite.objects.filter(user=user).exclude(
    #     Q(
    #         object_id__in=models.Workflow.objects.filter(
    #             Q(deleted=True) | Q(project__deleted=True)
    #         ),
    #         content_type=ContentType.objects.get_for_model(models.Workflow)
    #     )
    #     | Q(
    #         object_id__in=models.Project.objects.filter(deleted=True),
    #         content_type=ContentType.objects.get_for_model(models.Project)
    #     )
    # )


def check_possible_parent(workflow, parent_workflow, same_project):
    order = ["activity", "course", "program"]
    try:
        if order.index(workflow.type) == order.index(parent_workflow.type) - 1:
            if same_project:
                if workflow.get_project() == parent_workflow.get_project():
                    return True
            else:
                return True
    except IndexError:
        pass
    return False


def get_classrooms_for_student(user):
    return models.Project.objects.filter(
        liveproject__liveprojectuser__user=user,
    )


def get_user_permission(obj, user):
    if obj.type in ["workflow", "course", "activity", "program"]:
        obj = models.Workflow.objects.get(pk=obj.pk)

    if user is None or not user.is_authenticated:
        return models.ObjectPermission.PERMISSION_NONE
    if obj.author == user:
        return models.ObjectPermission.PERMISSION_EDIT
    permissions = models.ObjectPermission.objects.filter(
        user=user,
        content_type=ContentType.objects.get_for_model(obj),
        object_id=obj.id,
    )
    if permissions.count() == 0:
        return models.ObjectPermission.PERMISSION_NONE
    return permissions.first().permission_type


def get_user_role(obj, user):
    if user is None or not user.is_authenticated:
        return models.LiveProjectUser.ROLE_NONE
    if obj.type == "liveproject":
        liveproject = obj
        project = obj.project
    elif obj.type == "project":
        try:
            liveproject = obj.liveproject
            project = obj
        except AttributeError:
            return models.LiveProjectUser.ROLE_NONE
    elif obj.is_strategy:
        project = None
        liveproject = None
    else:
        try:
            project = obj.get_project()
            liveproject = project.liveproject
        except AttributeError:
            return models.LiveProjectUser.ROLE_NONE
    if liveproject is None:
        return models.LiveProjectUser.ROLE_NONE
    if hasattr(obj,"author") and obj.author == user:
        return models.LiveProjectUser.ROLE_TEACHER
    permissions = models.LiveProjectUser.objects.filter(
        user=user, liveproject=liveproject
    )
    if permissions.count() == 0:
        return models.LiveProjectUser.ROLE_NONE
    return permissions.first().role_type


def user_workflow_url(workflow, user):
    user_permission = get_user_permission(workflow, user)
    user_role = get_user_role(workflow, user)
    can_view = False
    is_public = workflow.public_view
    if user is not None and user.is_authenticated and workflow.published:
        if Group.objects.get(name=settings.TEACHER_GROUP) in user.groups.all():
            can_view = True
    if user_permission != models.ObjectPermission.PERMISSION_NONE:
        can_view = True
    if user_role != models.LiveProjectUser.ROLE_NONE:
        can_view = True
    if can_view:
        return reverse(
            "course_flow:workflow-update", kwargs={"pk": workflow.pk}
        )
    if is_public:
        return reverse(
            "course_flow:workflow-public", kwargs={"pk": workflow.pk}
        )
    if user is None or not user.is_authenticated:
        return "nouser"
    return "noaccess"


def save_serializer(serializer) -> HttpResponse:
    if serializer:
        if serializer.is_valid():
            serializer.save()
            return JsonResponse({"action": "posted"})
        else:
            return JsonResponse({"action": "error"})
    else:
        return JsonResponse({"action": "error"})


def get_relevance(obj, name_filter, keywords):
    if obj.title is None:
        title = ""
    else:
        title = obj.title.lower()
    if obj.description is None:
        description = ""
    else:
        description = obj.description.lower()
    if obj.author is None:
        first = ""
        last = ""
        username = ""
    else:
        if obj.author.first_name is None:
            first = ""
        else:
            first = obj.author.first_name
        if obj.author.last_name is None:
            last = ""
        else:
            last = obj.author.last_name
        if obj.author.username is None:
            username = ""
        else:
            username = obj.author.username
    relevance = ""
    to_check = [name_filter] + keywords
    keys = [title, last, first, username, description]
    for key in keys:
        for keyword in to_check:
            if keyword == "":
                continue
            if key.startswith(keyword):
                relevance += "0"
            elif key.find(" " + keyword) >= 0:
                relevance += "1"
            else:
                relevance += "2"
    return relevance


def benchmark(identifier, last_time):
    current_time = time.time()
    print("Completed " + identifier + " in " + str(current_time - last_time))
    return current_time



# Generated by Django 2.2.20 on 2021-06-21 21:54

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0058_auto_20210506_2247"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="outcomes",
        ),
        migrations.AlterField(
            model_name="outcomehorizontallink",
            name="outcome",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="outcome_horizontal_links",
                to="course_flow.Outcome",
            ),
        ),
        migrations.AlterField(
            model_name="outcomehorizontallink",
            name="parent_outcome",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="reverse_outcome_horizontal_links",
                to="course_flow.Outcome",
            ),
        ),
        migrations.DeleteModel(
            name="OutcomeProject",
        ),
    ]



import json

from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer

from .decorators import check_object_permission
from .models import ObjectPermission, Workflow


class WorkflowUpdateConsumer(WebsocketConsumer):
    def get_permission(self):
        workflow = Workflow.objects.get(pk=self.workflow_pk)
        self.VIEW = check_object_permission(
            workflow, self.user, ObjectPermission.PERMISSION_VIEW
        )
        self.EDIT = check_object_permission(
            workflow, self.user, ObjectPermission.PERMISSION_EDIT
        )

    def connect(self):
        self.workflow_pk = self.scope["url_route"]["kwargs"]["workflowPk"]
        self.room_group_name = "workflow_" + self.workflow_pk
        self.user = self.scope["user"]

        try:
            self.get_permission()
        except Exception:
            return self.close()
        if self.VIEW or self.EDIT:
            async_to_sync(self.channel_layer.group_add)(
                self.room_group_name, self.channel_name
            )
            return self.accept()
        return self.close()

    def disconnect(self, close_code):
        try:
            async_to_sync(self.channel_layer.group_send)(
                self.room_group_name,
                {"type": "lock_update", "action": self.last_lock},
            )
        except AttributeError:
            pass
        async_to_sync(self.channel_layer.group_discard)(
            self.room_group_name, self.channel_name
        )
        # super().disconnect()
        # self.close()

    def receive(self, text_data):
        if not self.EDIT:
            return
        text_data_json = json.loads(text_data)

        if text_data_json["type"] == "micro_update":
            action = text_data_json["action"]
            async_to_sync(self.channel_layer.group_send)(
                self.room_group_name,
                {"type": "workflow_action", "action": action},
            )
        elif text_data_json["type"] == "lock_update":
            lock = text_data_json["lock"]
            if lock["lock"]:
                self.last_lock = {**lock, "lock": False}
            async_to_sync(self.channel_layer.group_send)(
                self.room_group_name, {"type": "lock_update", "action": lock}
            )
        elif text_data_json["type"] == "connection_update":
            user_data = text_data_json["user_data"]
            async_to_sync(self.channel_layer.group_send)(
                self.room_group_name,
                {"type": "connection_update", "action": user_data},
            )

    def workflow_action(self, event):
        if not self.VIEW:
            return
        # Send message to WebSocket
        if event["type"] == "workflow_action":
            self.send(text_data=json.dumps(event))

    def lock_update(self, event):
        if not self.VIEW:
            return
        # Send message to WebSocket
        if event["type"] == "lock_update":
            self.send(text_data=json.dumps(event))

    def connection_update(self, event):
        if not self.VIEW:
            return
        # Send message to WebSocket
        if event["type"] == "connection_update":
            self.send(text_data=json.dumps(event))

    def workflow_parent_updated(self, event):
        if not self.VIEW:
            return
        # Send message to WebSocket
        if event["type"] == "workflow_parent_updated":
            self.send(text_data=json.dumps(event))

    def workflow_child_updated(self, event):
        if not self.VIEW:
            return
        # Send message to WebSocket
        if event["type"] == "workflow_child_updated":
            self.send(text_data=json.dumps(event))



# Generated by Django 3.2.15 on 2023-02-05 16:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0102_auto_20230205_1612"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="activity",
            name="author",
        ),
        migrations.RemoveField(
            model_name="course",
            name="author",
        ),
        migrations.RemoveField(
            model_name="program",
            name="author",
        ),
    ]



from django.apps import AppConfig
from django.core.checks import register

from .checks import check_return_url


class CourseFlowConfig(AppConfig):
    name = "course_flow"
    verbose_name = "Course Flow"

    def ready(self):
        register(check_return_url)



# Generated by Django 3.2.15 on 2023-04-09 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('course_flow', '0111_alter_favourite_content_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='column',
            name='icon',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
    ]



from django.core.management.base import BaseCommand

from course_flow.models import Discipline


class Command(BaseCommand):
    def add_arguments(self, parser):
        pass

    def handle(self, *args, **kwargs):
        for title in [
            "Physics",
            "Chemistry",
            "Biology",
            "Environmental Science",
            "Science (General)",
            "Mathematics",
            "Philosophy",
            "Computer Science",
            "Engineering",
            "Anthropology",
            "Economics",
            "Geography",
            "Political Science",
            "Psychology",
            "Sociology",
            "Social Work",
            "Social Sciences (General)",
            "Performing Arts",
            "Visual Arts",
            "History",
            "Literature",
            "Law",
            "Theology",
            "Humanities (General)",
            "Medicine",
            "Nursing",
            "Business",
            "English",
            "French",
            "Languages",
            "Design",
            "Other",
        ]:
            try:
                Discipline.objects.get(title=title)
            except Exception:
                Discipline.objects.create(title=title)



# Generated by Django 3.2.14 on 2022-09-30 19:41

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0089_auto_20220930_1527"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="hash",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, null=True
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2022-09-21 15:55

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0087_auto_20220704_1828"),
    ]

    operations = [
        migrations.CreateModel(
            name="LiveProject",
            fields=[
                (
                    "created_on",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "project",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="course_flow.project",
                    ),
                ),
                ("default_self_reporting", models.BooleanField(default=True)),
                ("default_assign_to_all", models.BooleanField(default=True)),
                (
                    "default_single_completion",
                    models.BooleanField(default=False),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.14 on 2022-10-04 15:26

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0092_auto_20220930_1948"),
    ]

    operations = [
        migrations.CreateModel(
            name="LiveProjectUser",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "role_type",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "None"),
                            (1, "Student"),
                            (2, "Instructor"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "liveproject",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.liveproject",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



import os
from functools import wraps

import celery
from celery.utils.log import get_logger

logger = get_logger("peerinst-scheduled")

# Set the default Django settings module for the 'celery' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "course_flow.settings")

app = celery.Celery("course_flow")

# Using a string here means the worker doesn't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object("django.conf:settings", namespace="CELERY")

# Load task modules from all registered Django apps.
app.autodiscover_tasks()


@app.task
def heartbeat():
    logger.info("Heartbeat check")
    pass


def try_async(func):
    """
    From https://github.com/SALTISES4/dalite-ng/blob/master/dalite/celery.py
    Decorator for celery tasks such that they default to synchronous operation
    if no workers are available
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            heartbeat.delay()

        except heartbeat.OperationalError as e:
            info = "Celery unavailable ({}).  Executing {} synchronously.".format(  # noqa
                e, func.__name__
            )
            logger.info(info)
            return func(*args, **kwargs)

        else:
            logger.info("Checking for available workers...")
            available_workers = celery.current_app.control.inspect().active()

            if available_workers:
                info = "Celery workers available ({}).  Executing {} asynchronously.".format(  # noqa
                    list(available_workers.keys()), func.__name__
                )
                logger.info(info)
                return func.delay(*args, **kwargs)
            else:
                info = "No celery workers available.  Executing {} synchronously.".format(  # noqa
                    func.__name__
                )
                logger.info(info)
                return func(*args, **kwargs)

    return wrapper



# Generated by Django 2.2.16 on 2021-03-15 19:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="course",
            name="discipline",
        ),
        migrations.AddField(
            model_name="project",
            name="discipline",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Discipline",
            ),
        ),
    ]



# Generated by Django 2.2.20 on 2021-04-27 22:28

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0055_delete_outcomeworkflow"),
    ]

    operations = [
        migrations.CreateModel(
            name="OutcomeWorkflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "outcome",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Outcome",
                    ),
                ),
                (
                    "workflow",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Workflow",
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome-Workflow Link",
                "verbose_name_plural": "Outcome-Workflow Links",
            },
        ),
        migrations.AddField(
            model_name="workflow",
            name="outcomes",
            field=models.ManyToManyField(
                blank=True,
                through="course_flow.OutcomeWorkflow",
                to="course_flow.Outcome",
            ),
        ),
    ]



# Generated by Django 2.2.20 on 2021-07-12 18:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0060_auto_20210621_2207"),
    ]

    operations = [
        migrations.AddField(
            model_name="outcomehorizontallink",
            name="degree",
            field=models.PositiveIntegerField(default=1),
        ),
    ]



import json
from functools import reduce, wraps

from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.db.models import Q
from django.http import (
    HttpResponseBadRequest,
    HttpResponseNotFound,
    JsonResponse,
)
from django.views.decorators.http import require_GET, require_POST
from ratelimit.decorators import ratelimit

from course_flow.models import (
    LiveProjectUser,
    ObjectPermission,
    User,
    Workflow,
)
from course_flow.utils import get_model_from_str


# Ajax login required view decorator
def ajax_login_required(view_func):
    """
    Decorator for ajax views that checks if a user is logged in and returns
    a 401 status code otherwise.
    """

    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        if request.user.is_authenticated:
            return view_func(request, *args, **kwargs)
        else:
            response = JsonResponse({"login_url": settings.LOGIN_URL})
            response.status_code = 401
            return response

    return _wrapped_view


# Ajax login required view decorator
def ajax_login_as_teacher_required(view_func):
    """
    Decorator for ajax views that checks if a user is logged in and returns
    a 401 status code otherwise.
    """

    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        if request.user.is_authenticated and (
            Group.objects.get(name=settings.TEACHER_GROUP)
            in request.user.groups.all()
        ):
            return view_func(request, *args, **kwargs)
        else:
            response = JsonResponse({"login_url": settings.LOGIN_URL})
            response.status_code = 401
            return response

    return _wrapped_view


def is_owner(model):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(request, model=model, *args, **kwargs):
            if model:
                if model[-2:] == "Pk":
                    pk = json.loads(request.POST.get(model, json.dumps("")))
                    model = model[:-2]
            else:
                pk = json.loads(request.POST.get("objectID", json.dumps("")))
                model = json.loads(
                    request.POST.get("objectType", json.dumps(""))
                )

            if not pk or not model:
                return HttpResponseBadRequest()

            try:
                object_type = get_model_from_str(model)
                object = object_type.objects.get(pk=pk)
            except ObjectDoesNotExist:
                return HttpResponseNotFound()

            # Check ownership
            if request.user == object.author:
                return fct(request, *args, **kwargs)
            else:
                raise PermissionDenied

        return _wrapped_view

    return wrapped_view


def check_object_permission(instance, user, permission):
    if isinstance(instance, Workflow):
        instance = Workflow.objects.get(pk=instance.pk)
    object_permission = ObjectPermission.objects.filter(
        user=user,
        object_id=instance.pk,
        content_type=ContentType.objects.get_for_model(instance),
    )
    if instance.author == user:
        if object_permission.count() == 0:
            ObjectPermission.objects.create(
                user=user,
                content_object=instance,
                permission_type=ObjectPermission.PERMISSION_EDIT,
            )
        return True
    if permission == ObjectPermission.PERMISSION_VIEW:
        if instance.published:
            return True
        permission_check = (
            Q(permission_type=ObjectPermission.PERMISSION_EDIT)
            | Q(permission_type=ObjectPermission.PERMISSION_VIEW)
            | Q(permission_type=ObjectPermission.PERMISSION_COMMENT)
        )
    elif permission == ObjectPermission.PERMISSION_COMMENT:
        permission_check = Q(
            permission_type=ObjectPermission.PERMISSION_EDIT
        ) | Q(permission_type=ObjectPermission.PERMISSION_COMMENT)
    else:
        permission_check = Q(permission_type=permission)
    if object_permission.filter(permission_check).count() > 0:
        return True


def check_objects_permission(instances, user, permission):
    object_permissions = [
        check_object_permission(x, user, permission) for x in instances
    ]
    return reduce(lambda a, b: a | b, object_permissions)


def check_objects_public(instances):
    object_public = [x.public_view for x in instances]
    return reduce(lambda a, b: a & b, object_public)


def check_special_case_delete_permission(model_data, user):
    instance = get_model_from_str(model_data["model"]).objects.get(
        pk=model_data["pk"]
    )
    #    if (
    #        model_data["model"] == "outcome"
    #        and OutcomeWorkflow.objects.filter(outcome=instance).count() == 0
    #    ):
    #        permission_objects = instance.get_permission_objects()
    #        return check_objects_permission(
    #            permission_objects, user, ObjectPermission.PERMISSION_EDIT
    #        )
    if model_data["model"] == "project":
        return instance.author == user
    else:
        if instance.get_project() is None:
            return instance.author == user
        return instance.author == user or check_object_permission(instance.get_project(),user,ObjectPermission.PERMISSION_EDIT)


def get_model_from_request(model, request, **kwargs):
    if model:
        if model[-2:] == "Pk":
            request_data = request.POST.get(model)
            if request_data is None:
                pk = None
            else:
                pk = json.loads(request.POST.get(model))
            model = model[:-2]
    else:
        get_parent = kwargs.get("get_parent", False)
        if get_parent:
            request_data = request.POST.get("parentID")
            if request_data is None:
                pk = None
                model = None
            else:
                pk = json.loads(request_data)
                model = json.loads(request.POST.get("parentType"))
        else:
            request_data = request.POST.get("objectID")
            if request_data is None:
                pk = None
                model = None
            else:
                pk = json.loads(request_data)
                model = json.loads(request.POST.get("objectType"))
    return {"model": model, "pk": pk}


def get_permission_objects(model, request, **kwargs):
    model_data = get_model_from_request(model, request, **kwargs)
    object_type = get_model_from_str(model_data["model"])
    permission_objects = object_type.objects.get(
        pk=model_data["pk"]
    ).get_permission_objects()
    return permission_objects


def user_is_author(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
                if all(
                    [
                        obj.author == User.objects.get(pk=request.user.pk)
                        for obj in permission_objects
                    ]
                ):
                    return fct(request, *args, **kwargs)
                else:
                    response = JsonResponse({"login_url": settings.LOGIN_URL})
                    response.status_code = 403
                    return response
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_edit(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
                if check_objects_permission(
                    permission_objects,
                    User.objects.get(pk=request.user.pk),
                    ObjectPermission.PERMISSION_EDIT,
                ):
                    return fct(request, *args, **kwargs)
                else:
                    response = JsonResponse({"login_url": settings.LOGIN_URL})
                    response.status_code = 403
                    return response
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_view(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_VIEW,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_view_or_none(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                model_data = get_model_from_request(
                    model, request, **outer_kwargs
                )
                if model_data["pk"] is None or model_data["pk"] == -1:
                    return fct(request, *args, **kwargs)
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_VIEW,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_edit_or_none(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                model_data = get_model_from_request(
                    model, request, **outer_kwargs
                )
                if model_data["pk"] is None or model_data["pk"] == -1:
                    return fct(request, *args, **kwargs)
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_EDIT,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


delete_exceptions = [
    "workflow",
    "activity",
    "course",
    "program",
    "project",
]


def user_can_comment(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_COMMENT,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_delete(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_as_teacher_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                model_data = get_model_from_request(
                    model, request, **outer_kwargs
                )
                if model_data["model"] in delete_exceptions:
                    try:
                        perm = check_special_case_delete_permission(
                            model_data, User.objects.get(pk=request.user.pk)
                        )
                    except Exception as e:
                        response = JsonResponse({"error": str(e)})
                        response.status_code = 400
                        return response
                    if perm:
                        return fct(request, *args, **kwargs)
                else:
                    permission_objects = get_permission_objects(
                        model, request, **outer_kwargs
                    )
                    if check_objects_permission(
                        permission_objects,
                        User.objects.get(pk=request.user.pk),
                        ObjectPermission.PERMISSION_EDIT,
                    ):
                        return fct(request, *args, **kwargs)

                response = JsonResponse({"error": "permission"})
                response.status_code = 403
                return response
            except Exception as e:
                response = JsonResponse({"error": str(e)})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


# check to see if the two models are from the same workflow. The second object may be parent if kwargs are used
# The second argument can be optional; if no model info is found the decorator passes
def from_same_workflow(model1, model2, **outer_kwargs):
    def wrapped_view(fct):
        @wraps(fct)
        def _wrapped_view(
            request,
            model1=model1,
            model2=model2,
            outer_kwargs=outer_kwargs,
            *args,
            **kwargs
        ):
            try:
                model_data1 = get_model_from_request(
                    model1,
                    request,
                )
                model_data2 = get_model_from_request(
                    model2, request, **outer_kwargs
                )
                if json.loads(
                    request.POST.get("allowDifferent", "false")
                ) and not json.loads(
                    request.POST.get("columnChange", "false")
                ):
                    return fct(request, *args, **kwargs)
                if model_data2["pk"] is None or model_data2["pk"] == -1:
                    return fct(request, *args, **kwargs)
                instance1 = get_model_from_str(
                    model_data1["model"]
                ).objects.get(pk=model_data1["pk"])
                instance2 = get_model_from_str(
                    model_data2["model"]
                ).objects.get(pk=model_data2["pk"])
                if instance1.get_workflow().pk == instance2.get_workflow().pk:
                    return fct(request, *args, **kwargs)
                response = JsonResponse({"error": "workflow mismatch"})
                response.status_code = 403
                return response
            except Exception as e:
                response = JsonResponse({"error": str(e)})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_is_teacher():
    def wrapped_view(fct):
        @require_POST
        @ajax_login_required
        @wraps(fct)
        def _wrapped_view(request, *args, **kwargs):
            try:
                if (
                    Group.objects.get(name=settings.TEACHER_GROUP)
                    in request.user.groups.all()
                ):
                    return fct(request, *args, **kwargs)
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def public_model_access(model, **outer_kwargs):
    rate_per_min = outer_kwargs.get("rate", 5)

    def wrapped_view(fct):
        @require_GET
        @ratelimit(key="ip", rate=str(rate_per_min) + "/m", method=["GET"])
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            ratelimited = getattr(request, "limited", False)
            if ratelimited:
                response = JsonResponse({"action": "ratelimited"})
                response.status_code = 429
                return response
            try:
                model_type = get_model_from_str(model)
                permission_objects = model_type.objects.get(
                    pk=kwargs.get("pk")
                ).get_permission_objects()
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_public(permission_objects):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


# Live project decorators


def get_enrollment_objects(model, request, **kwargs):
    model_data = get_model_from_request(model, request, **kwargs)
    object_type = get_model_from_str(model_data["model"])
    permission_objects = [
        object_type.objects.get(pk=model_data["pk"]).get_live_project()
    ]
    return permission_objects


def check_object_enrollment(instance, user, role):
    if instance.type == "liveproject":
        liveproject = instance
    elif instance.type == "project":
        liveproject = instance.liveproject
    elif instance.type in ["activity", "course", "program"]:
        try:
            liveproject = instance.get_project().liveproject
        except AttributeError:
            return False
        if user != liveproject.project.author:
            if (
                liveproject.visible_workflows.filter(
                    pk=instance.pk, deleted=False
                ).count()
                == 0
            ):
                return False
    if liveproject is None:
        return False
    if liveproject.project.author == user:
        return True

    if role == LiveProjectUser.ROLE_STUDENT:
        permission_check = Q(role_type=LiveProjectUser.ROLE_STUDENT) | Q(
            role_type=LiveProjectUser.ROLE_TEACHER
        )
    else:
        permission_check = Q(role_type=role)
    if (
        LiveProjectUser.objects.filter(user=user, liveproject=liveproject)
        .filter(permission_check)
        .count()
        > 0
    ):
        return True


def check_objects_enrollment(instances, user, role):
    object_permissions = [
        check_object_enrollment(x, user, role) for x in instances
    ]
    return reduce(lambda a, b: a | b, object_permissions)


def user_enrolled_as_teacher(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_enrollment(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                LiveProjectUser.ROLE_TEACHER,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_enrolled_as_student(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_enrollment(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                LiveProjectUser.ROLE_STUDENT,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_view_or_enrolled_as_student(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_enrollment(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                LiveProjectUser.ROLE_STUDENT,
            ) or check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_VIEW,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view


def user_can_view_or_enrolled_as_teacher(model, **outer_kwargs):
    def wrapped_view(fct):
        @require_POST
        @ajax_login_required
        @wraps(fct)
        def _wrapped_view(
            request, model=model, outer_kwargs=outer_kwargs, *args, **kwargs
        ):
            try:
                permission_objects = get_permission_objects(
                    model, request, **outer_kwargs
                )
            except AttributeError:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response
            if check_objects_enrollment(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                LiveProjectUser.ROLE_TEACHER,
            ) or check_objects_permission(
                permission_objects,
                User.objects.get(pk=request.user.pk),
                ObjectPermission.PERMISSION_VIEW,
            ):
                return fct(request, *args, **kwargs)
            else:
                response = JsonResponse({"login_url": settings.LOGIN_URL})
                response.status_code = 403
                return response

        return _wrapped_view

    return wrapped_view



# Generated by Django 3.2.15 on 2023-01-18 21:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0099_alter_courseflowuser_user"),
    ]

    operations = [
        migrations.AddField(
            model_name="courseflowuser",
            name="notifications",
            field=models.PositiveIntegerField(
                choices=[(0, "Unknown"), (1, "No"), (2, "Yes")], default=0
            ),
        ),
    ]



# Generated by Django 2.2.25 on 2022-01-12 22:55

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0077_workflow_condensed"),
    ]

    operations = [
        migrations.AddField(
            model_name="column",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="node",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="nodelink",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="outcome",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="project",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="week",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="workflow",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



from django.core.management.base import BaseCommand

from course_flow.models import Column, Node, NodeLink, Outcome, Week, Workflow


class Command(BaseCommand):
    def add_arguments(self, parser):
        pass

    def handle(self, *args, **kwargs):
        # Deletes all the orphaned material. This can be produced when copying/importing is interrupted. Use with extreme caution.
        orphaned_nodes = Node.objects.filter(nodeweek=None)
        print(
            "There are "
            + str(orphaned_nodes.count())
            + " orphaned nodes in existence"
        )
        orphaned_nodes.delete()
        orphaned_nodelinks = NodeLink.objects.filter(source_node=None)
        print(
            "There are "
            + str(orphaned_nodelinks.count())
            + " orphaned nodelinks in existence"
        )
        orphaned_nodelinks.delete()
        orphaned_weeks = Week.objects.filter(weekworkflow=None)
        print(
            "There are "
            + str(orphaned_weeks.count())
            + " orphaned weeks in existence"
        )
        orphaned_weeks.delete()
        orphaned_columns = Column.objects.filter(columnworkflow=None)
        print(
            "There are "
            + str(orphaned_columns.count())
            + " orphaned columns in existence"
        )
        orphaned_columns.delete()
        orphaned_outcomes = Outcome.objects.filter(
            outcomeworkflow=None, parent_outcome_links=None
        )
        orphaned_workflows = Workflow.objects.filter(
            workflowproject=None, is_strategy=False
        )
        print(
            "There are "
            + str(orphaned_workflows.count())
            + " orphaned workflows in existence"
        )
        print(
            "There are "
            + str(orphaned_outcomes.count())
            + " orphaned outcomes in existence"
        )
        orphaned_outcomes.delete()
        orphaned_workflows.delete()



from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.db.models import F

from .models import Node


def dispatch_wf(workflow, action):
    workflow.edit_count = F("edit_count") + 1
    workflow.save()
    workflow.refresh_from_db()
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        "workflow_" + str(workflow.pk),
        {
            "type": "workflow_action",
            "action": action,
            "edit_count": str(workflow.edit_count),
        },
    )


def dispatch_to_parent_wf(workflow, action):
    channel_layer = get_channel_layer()
    for parent_node in Node.objects.filter(linked_workflow=workflow):
        parent_workflow = parent_node.get_workflow()
        parent_workflow.edit_count = F("edit_count") + 1
        parent_workflow.save()
        parent_workflow.refresh_from_db()
        async_to_sync(channel_layer.group_send)(
            "workflow_" + str(parent_workflow.pk),
            {
                "type": "workflow_action",
                "action": action,
                "edit_count": parent_workflow.edit_count,
            },
        )


def dispatch_parent_updated(workflow):
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        "workflow_" + str(workflow.pk),
        {
            "type": "workflow_parent_updated",
            "edit_count": workflow.edit_count,
        },
    )


def dispatch_child_updated(workflow, child_workflow):
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        "workflow_" + str(workflow.pk),
        {
            "type": "workflow_child_updated",
            "edit_count": workflow.edit_count,
            "child_workflow_id": child_workflow.pk,
        },
    )


def dispatch_wf_lock(workflow, action):
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        "workflow_" + str(workflow.pk),
        {"type": "lock_update", "action": action},
    )


# def dispatch_wf(workflow, action):
#    workflow.edit_count = F("edit_count") + 1
#    workflow.save()
#    workflow.refresh_from_db()
#    channel_layer = get_channel_layer()
#    channel_layer.group_send(
#        "workflow_" + str(workflow.pk),
#        {
#            "type": "workflow_action",
#            "action": action,
#            "edit_count": str(workflow.edit_count),
#        },
#    )
#
#
# def dispatch_to_parent_wf(workflow, action):
#    channel_layer = get_channel_layer()
#    for parent_node in Node.objects.filter(linked_workflow=workflow):
#        parent_workflow = parent_node.get_workflow()
#        parent_workflow.edit_count = F("edit_count") + 1
#        parent_workflow.save()
#        parent_workflow.refresh_from_db()
#        channel_layer.group_send(
#            "workflow_" + str(parent_workflow.pk),
#            {
#                "type": "workflow_action",
#                "action": action,
#                "edit_count": parent_workflow.edit_count,
#            },
#        )
#
#
# def dispatch_wf_lock(workflow, action):
#    channel_layer = get_channel_layer()
#    channel_layer.group_send(
#        "workflow_" + str(workflow.pk),
#        {"type": "lock_update", "action": action},
#    )


# Actions for reduers
def unlock(object_id, object_type):
    return {
        "lock": False,
        "object_id": object_id,
        "object_type": object_type,
    }


def changeThroughID(through_type, old_id, new_id, extra_data):
    return {
        "type": through_type + "/changeID",
        "payload": {"old_id": old_id, "new_id": new_id, **extra_data},
    }


def deleteSelfAction(id, objectType, parentID, extra_data):
    return {
        "type": objectType + "/deleteSelf",
        "payload": {"id": id, "parent_id": parentID, "extra_data": extra_data},
    }


def deleteSelfSoftAction(id, objectType, parentID, extra_data):
    return {
        "type": objectType + "/deleteSelfSoft",
        "payload": {"id": id, "parent_id": parentID, "extra_data": extra_data},
    }


def restoreSelfAction(
    id, objectType, parentID, throughparentID, throughparent_index, extra_data
):
    return {
        "type": objectType + "/restoreSelf",
        "payload": {
            "id": id,
            "parent_id": parentID,
            "throughparent_id": throughparentID,
            "throughparent_index": throughparent_index,
            "extra_data": extra_data,
        },
    }


def insertBelowAction(response_data, objectType):
    return {"type": objectType + "/insertBelow", "payload": response_data}


def insertChildAction(response_data, objectType):
    return {"type": objectType + "/insertChild", "payload": response_data}


def setLinkedWorkflowAction(response_data):
    return {"type": "node/setLinkedWorkflow", "payload": response_data}


def newNodeAction(response_data):
    return {"type": "node/newNode", "payload": response_data}


def newOutcomeAction(response_data):
    return {"type": "outcome/newOutcome", "payload": response_data}


def newNodeLinkAction(response_data):
    return {"type": "nodelink/newNodeLink", "payload": response_data}


def changeField(id, objectType, json, changeFieldID=0):
    return {
        "type": objectType + "/changeField",
        "payload": {
            "id": id,
            "objectType": objectType,
            "json": json,
            "changeFieldID": changeFieldID,
        },
    }


def changeFieldMany(ids, objectType, json, changeFieldID=0):
    return {
        "type": objectType + "/changeFieldMany",
        "payload": {
            "ids": ids,
            "objectType": objectType,
            "json": json,
            "changeFieldID": changeFieldID,
        },
    }


def updateOutcomenodeDegreeAction(response_data):
    return {"type": "outcomenode/updateDegree", "payload": response_data}


def updateOutcomehorizontallinkDegreeAction(response_data):
    return {
        "type": "outcomehorizontallink/updateDegree",
        "payload": response_data,
    }


def updateChildOutcomehorizontallinkDegreeAction(response_data):
    return {
        "type": "childoutcomehorizontallink/updateDegree",
        "payload": response_data,
    }


def newStrategyAction(response_data):
    return {"type": "strategy/addStrategy", "payload": response_data}


def toggleStrategyAction(response_data):
    return {"type": "strategy/toggleStrategy", "payload": response_data}


def gridMenuItemAdded(response_data):
    return {"type": "gridmenu/itemAdded", "payload": response_data}


def replaceStoreData(data_package):
    return {"type": "replaceStoreData", "payload": data_package}


def refreshStoreData(data_package):
    return {"type": "refreshStoreData", "payload": data_package}


def updateHorizontalLinks(data_package):
    return {"type": "outcome/updateHorizontalLinks", "payload": data_package}



# Generated by Django 2.2.20 on 2021-06-21 22:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0059_auto_20210621_2154"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="outcome",
            name="disciplines",
        ),
        migrations.RemoveField(
            model_name="outcome",
            name="published",
        ),
    ]



# Generated by Django 2.2.20 on 2021-12-10 19:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0073_nodelink_deleted"),
    ]

    operations = [
        migrations.AlterField(
            model_name="comment",
            name="text",
            field=models.CharField(max_length=2000),
        ),
        migrations.AlterField(
            model_name="node",
            name="description",
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="outcome",
            name="description",
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="outcome",
            name="title",
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="project",
            name="description",
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="week",
            name="description",
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="description",
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
    ]



# Generated by Django 2.2.25 on 2022-05-20 18:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0083_auto_20220322_2153"),
    ]

    operations = [
        migrations.AddField(
            model_name="week",
            name="is_dropped",
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 3.2.15 on 2022-11-18 15:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0095_updatenotification"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="node",
            name="students",
        ),
        migrations.DeleteModel(
            name="NodeCompletionStatus",
        ),
    ]



from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.urls import include, path

from . import settings, urls, views

app_name = "course_flow"

if settings.DEBUG:
    import debug_toolbar


def auth_patterns():
    return [
        path("register/", views.registration_view, name="registration"),
        path(
            "login/",
            auth_views.LoginView.as_view(
                template_name="course_flow/registration/login.html"
            ),
            name="login",
        ),
        path("logout/", auth_views.LogoutView.as_view(), name="logout"),
    ]


def app_patterns():
    patterns = [
        path(
            "course-flow/",
            include(
                (urls.urlpatterns, urls.app_name),
                namespace="course_flow",
            ),
        ),
        # path(
        #     "feedback/",
        #     include("user_feedback.urls", namespace="user_feedback"),
        # ),
        path("admin/", admin.site.urls),
    ]
    if settings.DEBUG:
        patterns += [path("__debug__/", include(debug_toolbar.urls))]
    return patterns


urlpatterns = sum(
    [auth_patterns(), app_patterns()],
    [],
)



# Generated by Django 3.2.15 on 2023-02-05 19:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0105_auto_20230205_1753"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="is_strategy",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.2.20 on 2021-11-23 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0072_auto_20211115_2237"),
    ]

    operations = [
        migrations.AddField(
            model_name="nodelink",
            name="deleted",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.15 on 2023-01-18 14:13

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0097_userassignment_completed_on"),
    ]

    operations = [
        migrations.CreateModel(
            name="CourseFlowUser",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="auth.user",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
                (
                    "last_name",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="program",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="authored_programs",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="authored_projects",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2023-09-11 18:41

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0113_project_from_saltise"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="course",
            options={
                "verbose_name": "Course",
                "verbose_name_plural": "Courses",
            },
        ),
        migrations.AlterModelOptions(
            name="discipline",
            options={
                "verbose_name": "Discipline",
                "verbose_name_plural": "Disciplines",
            },
        ),
        migrations.AlterModelOptions(
            name="node",
            options={"verbose_name": "Node", "verbose_name_plural": "Nodes"},
        ),
        migrations.AlterModelOptions(
            name="program",
            options={
                "verbose_name": "Program",
                "verbose_name_plural": "Programs",
            },
        ),
        migrations.AlterModelOptions(
            name="workflow",
            options={
                "verbose_name": "Workflow",
                "verbose_name_plural": "Workflows",
            },
        ),
        migrations.CreateModel(
            name="Notification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "notification_type",
                    models.PositiveIntegerField(
                        choices=[(0, "Shared"), (1, "Comment")], default=0
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                ("text", models.TextField(blank=True)),
                (
                    "created_on",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        limit_choices_to={
                            "model__in": ["project", "workflow"]
                        },
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "source_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="sent_notifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.14 on 2022-09-30 19:48

import uuid

from django.db import migrations


def gen_uuid(apps, schema_editor):
    Project = apps.get_model("course_flow", "project")
    for row in Project.objects.all():
        row.hash = uuid.uuid4()
        row.save(update_fields=["hash"])


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0090_project_hash"),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 2.2.17 on 2021-03-22 04:30

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Column",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("published", models.BooleanField(default=False)),
                ("visible", models.BooleanField(default=True)),
                ("colour", models.PositiveIntegerField(null=True)),
                (
                    "column_type",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "Custom Activity Column"),
                            (1, "Out of Class (Instructor)"),
                            (2, "Out of Class (Students)"),
                            (3, "In Class (Instructor)"),
                            (4, "In Class (Students)"),
                            (10, "Custom Course Column"),
                            (11, "Preparation"),
                            (12, "Lesson"),
                            (13, "Artifact"),
                            (14, "Assessment"),
                            (20, "Custom Program Category"),
                        ],
                        default=0,
                    ),
                ),
                ("is_original", models.BooleanField(default=False)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_column",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.Column",
                    ),
                ),
            ],
            options={
                "verbose_name": "Column",
                "verbose_name_plural": "Columns",
            },
        ),
        migrations.CreateModel(
            name="ColumnWorkflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "column",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Column",
                    ),
                ),
            ],
            options={
                "verbose_name": "Column-Workflow Link",
                "verbose_name_plural": "Column-Workflow Links",
            },
        ),
        migrations.CreateModel(
            name="Discipline",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Enter the name of a new discipline.",
                        max_length=100,
                        unique=True,
                        verbose_name="Discipline name",
                    ),
                ),
            ],
            options={
                "verbose_name": "discipline",
                "verbose_name_plural": "disciplines",
            },
        ),
        migrations.CreateModel(
            name="Node",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "description",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("published", models.BooleanField(default=False)),
                ("is_original", models.BooleanField(default=True)),
                ("has_autolink", models.BooleanField(default=False)),
                ("is_dropped", models.BooleanField(default=False)),
                (
                    "context_classification",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "None"),
                            (1, "Individual Work"),
                            (2, "Work in Groups"),
                            (3, "Whole Class"),
                            (101, "Formative"),
                            (102, "Summative"),
                            (103, "Comprehensive"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "task_classification",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "None"),
                            (1, "Gather Information"),
                            (2, "Discuss"),
                            (3, "Problem Solve"),
                            (4, "Analyze"),
                            (5, "Assess/Review Peers"),
                            (6, "Debate"),
                            (7, "Game/Roleplay"),
                            (8, "Create/Design"),
                            (9, "Revise/Improve"),
                            (10, "Read"),
                            (11, "Write"),
                            (12, "Present"),
                            (13, "Experiment/Inquiry"),
                            (14, "Quiz/Test"),
                            (15, "Instructor Resource Curation"),
                            (16, "Instructor Orchestration"),
                            (17, "Instructor Evaluation"),
                            (18, "Other"),
                            (101, "Jigsaw"),
                            (102, "Peer Instruction"),
                            (103, "Case Studies"),
                            (104, "Gallery Walk"),
                            (105, "Reflective Writing"),
                            (106, "Two-Stage Exam"),
                            (107, "Toolkit"),
                            (108, "One Minute Paper"),
                            (109, "Distributed Problem Solving"),
                            (110, "Peer Assessment"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "node_type",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "Activity Node"),
                            (1, "Course Node"),
                            (2, "Program Node"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "time_required",
                    models.CharField(blank=True, max_length=30, null=True),
                ),
                (
                    "time_units",
                    models.PositiveIntegerField(
                        choices=[
                            (0, ""),
                            (1, "seconds"),
                            (2, "minutes"),
                            (3, "hours"),
                            (4, "days"),
                            (5, "weeks"),
                            (6, "months"),
                            (7, "yrs"),
                            (8, "credits"),
                        ],
                        default=0,
                    ),
                ),
                ("represents_workflow", models.BooleanField(default=False)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="authored_nodes",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "column",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to="course_flow.Column",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="NodeWeek",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "node",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Node",
                    ),
                ),
            ],
            options={
                "verbose_name": "Node-Week Link",
                "verbose_name_plural": "Node-Week Links",
            },
        ),
        migrations.CreateModel(
            name="Outcome",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=500)),
                ("description", models.TextField(max_length=500)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("published", models.BooleanField(default=False)),
                ("is_original", models.BooleanField(default=True)),
                ("is_dropped", models.BooleanField(default=True)),
                ("depth", models.PositiveIntegerField(default=0)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome",
                "verbose_name_plural": "Outcomes",
            },
        ),
        migrations.CreateModel(
            name="OutcomeProject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "outcome",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Outcome",
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome-Project Link",
                "verbose_name_plural": "Outcome-Project Links",
            },
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "description",
                    models.CharField(blank=True, max_length=500, null=True),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("published", models.BooleanField(default=False)),
                ("is_original", models.BooleanField(default=False)),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "outcomes",
                    models.ManyToManyField(
                        blank=True,
                        through="course_flow.OutcomeProject",
                        to="course_flow.Outcome",
                    ),
                ),
                (
                    "parent_project",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.Project",
                    ),
                ),
            ],
            options={
                "verbose_name": "Project",
                "verbose_name_plural": "Projects",
            },
        ),
        migrations.CreateModel(
            name="Week",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "description",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("default", models.BooleanField(default=False)),
                ("is_original", models.BooleanField(default=True)),
                ("published", models.BooleanField(default=False)),
                ("is_strategy", models.BooleanField(default=False)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "strategy_classification",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "None"),
                            (1, "Jigsaw"),
                            (2, "Peer Instruction"),
                            (3, "Case Studies"),
                            (4, "Gallery Walk"),
                            (5, "Reflective Writing"),
                            (6, "Two-Stage Exam"),
                            (7, "Toolkit"),
                            (8, "One Minute Paper"),
                            (9, "Distributed Problem Solving"),
                            (10, "Peer Assessment"),
                            (11, "Other"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "week_type",
                    models.PositiveIntegerField(
                        choices=[(0, "Part"), (1, "Week"), (2, "Term")],
                        default=0,
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "nodes",
                    models.ManyToManyField(
                        blank=True,
                        through="course_flow.NodeWeek",
                        to="course_flow.Node",
                    ),
                ),
            ],
            options={
                "verbose_name": "Week",
                "verbose_name_plural": "Weeks",
            },
        ),
        migrations.CreateModel(
            name="WeekWorkflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "week",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Week",
                    ),
                ),
            ],
            options={
                "verbose_name": "Week-Workflow Link",
                "verbose_name_plural": "Week-Workflow Links",
            },
        ),
        migrations.CreateModel(
            name="Workflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "description",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("static", models.BooleanField(default=False)),
                ("published", models.BooleanField(default=False)),
                ("is_strategy", models.BooleanField(default=False)),
                ("from_saltise", models.BooleanField(default=False)),
                ("is_original", models.BooleanField(default=True)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "outcomes_type",
                    models.PositiveIntegerField(
                        choices=[(0, "Normal"), (1, "Advanced")], default=0
                    ),
                ),
                (
                    "outcomes_sort",
                    models.PositiveIntegerField(
                        choices=[
                            (0, "Time"),
                            (1, "Category"),
                            (2, "Task"),
                            (3, "Context"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "columns",
                    models.ManyToManyField(
                        blank=True,
                        through="course_flow.ColumnWorkflow",
                        to="course_flow.Column",
                    ),
                ),
                (
                    "parent_workflow",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.Workflow",
                    ),
                ),
                (
                    "weeks",
                    models.ManyToManyField(
                        blank=True,
                        through="course_flow.WeekWorkflow",
                        to="course_flow.Week",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="WorkflowProject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Project",
                    ),
                ),
                (
                    "workflow",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Workflow",
                    ),
                ),
            ],
            options={
                "verbose_name": "Workflow-Project Link",
                "verbose_name_plural": "Workflow-Project Links",
            },
        ),
        migrations.AddField(
            model_name="weekworkflow",
            name="workflow",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="course_flow.Workflow",
            ),
        ),
        migrations.AddField(
            model_name="week",
            name="original_strategy",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Workflow",
            ),
        ),
        migrations.AddField(
            model_name="week",
            name="parent_week",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Week",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="workflows",
            field=models.ManyToManyField(
                blank=True,
                through="course_flow.WorkflowProject",
                to="course_flow.Workflow",
            ),
        ),
        migrations.CreateModel(
            name="OutcomeWorkflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "outcome",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Outcome",
                    ),
                ),
                (
                    "workflow",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Workflow",
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome-Workflow Link",
                "verbose_name_plural": "Outcome-Workflow Links",
            },
        ),
        migrations.AddField(
            model_name="outcomeproject",
            name="project",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="course_flow.Project",
            ),
        ),
        migrations.CreateModel(
            name="OutcomeOutcome",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                (
                    "child",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="parent_outcome_links",
                        to="course_flow.Outcome",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="child_outcome_links",
                        to="course_flow.Outcome",
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome-Outcome Link",
                "verbose_name_plural": "Outcome-Outcome Links",
            },
        ),
        migrations.CreateModel(
            name="OutcomeNode",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("added_on", models.DateTimeField(auto_now_add=True)),
                ("rank", models.PositiveIntegerField(default=0)),
                ("degree", models.PositiveIntegerField(default=1)),
                (
                    "node",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Node",
                    ),
                ),
                (
                    "outcome",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Outcome",
                    ),
                ),
            ],
            options={
                "verbose_name": "Outcome-Node Link",
                "verbose_name_plural": "Outcome-Node Links",
            },
        ),
        migrations.AddField(
            model_name="outcome",
            name="children",
            field=models.ManyToManyField(
                blank=True,
                related_name="parent_outcomes",
                through="course_flow.OutcomeOutcome",
                to="course_flow.Outcome",
            ),
        ),
        migrations.AddField(
            model_name="outcome",
            name="parent_outcome",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Outcome",
            ),
        ),
        migrations.AddField(
            model_name="nodeweek",
            name="week",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="course_flow.Week",
            ),
        ),
        migrations.CreateModel(
            name="NodeLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=100, null=True),
                ),
                ("published", models.BooleanField(default=False)),
                (
                    "source_port",
                    models.PositiveIntegerField(
                        choices=[(1, "e"), (2, "s"), (3, "w")], default=2
                    ),
                ),
                (
                    "target_port",
                    models.PositiveIntegerField(
                        choices=[(0, "n"), (1, "e"), (3, "w")], default=0
                    ),
                ),
                ("dashed", models.BooleanField(default=False)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("is_original", models.BooleanField(default=True)),
                (
                    "hash",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_nodelink",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.NodeLink",
                    ),
                ),
                (
                    "source_node",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="outgoing_links",
                        to="course_flow.Node",
                    ),
                ),
                (
                    "target_node",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="incoming_links",
                        to="course_flow.Node",
                    ),
                ),
            ],
            options={
                "verbose_name": "Node Link",
                "verbose_name_plural": "Node Links",
            },
        ),
        migrations.CreateModel(
            name="NodeCompletionStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("is_completed", models.BooleanField(default=False)),
                (
                    "node",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.Node",
                    ),
                ),
                (
                    "student",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Node Completion Status",
                "verbose_name_plural": "Node Completion Statuses",
            },
        ),
        migrations.AddField(
            model_name="node",
            name="linked_workflow",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Workflow",
            ),
        ),
        migrations.AddField(
            model_name="node",
            name="outcomes",
            field=models.ManyToManyField(
                blank=True,
                through="course_flow.OutcomeNode",
                to="course_flow.Outcome",
            ),
        ),
        migrations.AddField(
            model_name="node",
            name="parent_node",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Node",
            ),
        ),
        migrations.AddField(
            model_name="node",
            name="students",
            field=models.ManyToManyField(
                blank=True,
                related_name="assigned_nodes",
                through="course_flow.NodeCompletionStatus",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="columnworkflow",
            name="workflow",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="course_flow.Workflow",
            ),
        ),
        migrations.CreateModel(
            name="Program",
            fields=[
                (
                    "workflow_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="course_flow.Workflow",
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            bases=("course_flow.workflow",),
        ),
        migrations.CreateModel(
            name="Course",
            fields=[
                (
                    "workflow_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="course_flow.Workflow",
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="authored_courses",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "discipline",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.Discipline",
                    ),
                ),
                (
                    "students",
                    models.ManyToManyField(
                        blank=True,
                        related_name="assigned_courses",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            bases=("course_flow.workflow",),
        ),
        migrations.CreateModel(
            name="Activity",
            fields=[
                (
                    "workflow_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="course_flow.Workflow",
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="authored_activities",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "students",
                    models.ManyToManyField(
                        blank=True,
                        related_name="assigned_activities",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Activity",
                "verbose_name_plural": "Activities",
            },
            bases=("course_flow.workflow",),
        ),
    ]



# Generated by Django 2.2.25 on 2022-02-28 18:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0078_auto_20220112_2255"),
    ]

    operations = [
        migrations.AlterField(
            model_name="column",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="term",
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="translation",
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="translation_plural",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="node",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="objectpermission",
            name="permission_type",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "None"),
                    (1, "View"),
                    (2, "Edit"),
                    (3, "Comment"),
                ],
                default=0,
            ),
        ),
        migrations.AlterField(
            model_name="outcome",
            name="code",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="week",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="code",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



from django.contrib import admin

from .models import (
    Activity,
    Column,
    ColumnWorkflow,
    Comment,
    Course,
    Discipline,
    Favourite,
    LiveAssignment,
    LiveProject,
    LiveProjectUser,
    Node,
    NodeLink,
    NodeWeek,
    ObjectPermission,
    ObjectSet,
    Outcome,
    OutcomeHorizontalLink,
    OutcomeNode,
    OutcomeOutcome,
    OutcomeWorkflow,
    Program,
    Project,
    UpdateNotification,
    UserAssignment,
    Week,
    WeekWorkflow,
    Workflow,
    WorkflowProject,
)

admin.site.register(Node)
admin.site.register(Column)
admin.site.register(ColumnWorkflow)
admin.site.register(Week)
admin.site.register(Activity)
admin.site.register(Course)
admin.site.register(Program)
admin.site.register(NodeWeek)
admin.site.register(NodeLink)
admin.site.register(WeekWorkflow)
admin.site.register(Workflow)
admin.site.register(Project)
admin.site.register(WorkflowProject)
admin.site.register(Outcome)
admin.site.register(OutcomeNode)
admin.site.register(OutcomeOutcome)
admin.site.register(OutcomeWorkflow)
admin.site.register(ObjectPermission)
admin.site.register(ObjectSet)
admin.site.register(OutcomeHorizontalLink)
admin.site.register(Discipline)
admin.site.register(Favourite)
admin.site.register(Comment)
admin.site.register(LiveProject)
admin.site.register(LiveProjectUser)
admin.site.register(LiveAssignment)
admin.site.register(UserAssignment)
admin.site.register(UpdateNotification)



# Generated by Django 3.2.15 on 2023-02-19 14:31

import django.db.models.deletion
from django.db import migrations, models


def forwards_func(apps, schema_editor):
    Favourite = apps.get_model("course_flow", "Favourite")
    ContentType = apps.get_model("contenttypes", "ContentType")
    workflow_contenttype = ContentType.objects.get_for_model(
        apps.get_model("course_flow", "Workflow")
    )
    activity_contenttype = ContentType.objects.get_for_model(
        apps.get_model("course_flow", "Activity")
    )
    course_contenttype = ContentType.objects.get_for_model(
        apps.get_model("course_flow", "Course")
    )
    program_contenttype = ContentType.objects.get_for_model(
        apps.get_model("course_flow", "Program")
    )
    db_alias = schema_editor.connection.alias
    Favourite.objects.using(db_alias).filter(
        content_type=activity_contenttype
    ).update(content_type=workflow_contenttype)
    Favourite.objects.using(db_alias).filter(
        content_type=course_contenttype
    ).update(content_type=workflow_contenttype)
    Favourite.objects.using(db_alias).filter(
        content_type=program_contenttype
    ).update(content_type=workflow_contenttype)


def reverse_func(apps, schema_editor):
    Favourite = apps.get_model("course_flow", "Favourite")
    ContentType = apps.get_model("contenttypes", "ContentType")
    Activity = apps.get_model("course_flow", "Activity")
    Course = apps.get_model("course_flow", "Course")
    Program = apps.get_model("course_flow", "Program")
    workflow_contenttype = ContentType.objects.get_for_model(
        apps.get_model("course_flow", "Workflow")
    )
    activity_contenttype = ContentType.objects.get_for_model(Activity)
    course_contenttype = ContentType.objects.get_for_model(Course)
    program_contenttype = ContentType.objects.get_for_model(Program)
    db_alias = schema_editor.connection.alias

    Favourite.objects.using(db_alias).exclude(activity=None).update(
        content_type=activity_contenttype
    )
    Favourite.objects.using(db_alias).exclude(course=None).update(
        content_type=course_contenttype
    )
    Favourite.objects.using(db_alias).exclude(program=None).update(
        content_type=program_contenttype
    )


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("course_flow", "0109_objectpermission_last_viewed"),
    ]

    operations = [
        migrations.AlterField(
            model_name="favourite",
            name="content_type",
            field=models.ForeignKey(
                limit_choices_to={
                    "model__in": [
                        "project",
                        "activity",
                        "course",
                        "program",
                        "workflow",
                    ]
                },
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2023-02-05 17:53

from django.db import migrations


def forwards_func(apps, schema_editor):
    Activity = apps.get_model("course_flow", "Activity")
    Course = apps.get_model("course_flow", "Course")
    Program = apps.get_model("course_flow", "Program")
    Project = apps.get_model("course_flow", "Project")
    ObjectPermission = apps.get_model("course_flow", "ObjectPermission")
    ContentType = apps.get_model("contenttypes", "ContentType")
    db_alias = schema_editor.connection.alias

    for workflow in (
        list(Activity.objects.all())
        + list(Course.objects.all())
        + list(Program.objects.all())
    ):
        if workflow.author is None:
            continue
        ObjectPermission.objects.using(db_alias).filter(
            content_type=ContentType.objects.get_for_model(workflow),
            object_id=workflow.id,
            user=workflow.author,
        ).delete()
        ObjectPermission.objects.using(db_alias).create(
            content_type=ContentType.objects.get_for_model(workflow),
            object_id=workflow.id,
            user=workflow.author,
            permission_type=2,
        )
    for project in Project.objects.all():
        if project.author is None:
            continue
        ObjectPermission.objects.using(db_alias).filter(
            content_type=ContentType.objects.get_for_model(project),
            object_id=project.id,
            user=project.author,
        ).delete()
        ObjectPermission.objects.using(db_alias).create(
            content_type=ContentType.objects.get_for_model(project),
            object_id=project.id,
            user=project.author,
            permission_type=2,
        )
    op = ObjectPermission.objects.filter(
        user__id=1,
        object_id=1,
        content_type=ContentType.objects.get_for_model(
            apps.get_model("course_flow", "Project")
        ),
    )


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0104_rename_author_temp_workflow_author"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="activity",
            name="students",
        ),
        migrations.RemoveField(
            model_name="course",
            name="students",
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



# Generated by Django 3.2.15 on 2023-06-01 18:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0112_column_icon"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="from_saltise",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.15 on 2022-11-01 17:37

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import course_flow.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0093_liveprojectuser"),
    ]

    operations = [
        migrations.CreateModel(
            name="LiveAssignment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("self_reporting", models.BooleanField(default=True)),
                ("single_completion", models.BooleanField(default=False)),
                (
                    "start_date",
                    models.DateTimeField(
                        default=course_flow.models.default_start_date
                    ),
                ),
                (
                    "end_date",
                    models.DateTimeField(
                        default=course_flow.models.default_due_date
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "liveproject",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.liveproject",
                    ),
                ),
                (
                    "task",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="course_flow.node",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="UserAssignment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("completed", models.BooleanField(default=False)),
                (
                    "assignment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="course_flow.liveassignment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
